@@ -5,6 +5,7 @@ from typing import (
     AnyStr,
     Callable,
     Dict,
+    Iterable,
     List,
     NoReturn,
     Optional,
@@ -24,7 +25,11 @@ from .linked_list import LinkedList
 
 
 class Rule:
-    """Base Rule definition."""
+    """Definition for atomic operation that validates data.
+
+    :param name: Name of the rule
+    :param resolver: Callable function that receives exactly 1 param as the data that will be validated
+    """
 
     _resolve: Callable
     _name: AnyStr
@@ -60,7 +65,10 @@ class Rule:
 
 
 class RuleSet:
-    """Rule Set definition to apply a set of rules to a context info."""
+    """Rule Set definition to apply a set of rules to a context info.
+
+    :param name: Identifier name of the rule set
+    """
 
     _name: AnyStr
     _rules: LinkedList[Rule]
@@ -84,11 +92,31 @@ class RuleSet:
         """
 
         if rule.__hash__() in self._rule_hashes:
-            raise RuleSetConfigError(f"Rule '{rule.name}' was already configured in the RuleSet", )
+            raise RuleSetConfigError(f"Rule '{rule.name}' was already configured in the RuleSet")
 
         self._rules.add_last(rule)
         self._rule_hashes.add(rule.__hash__())
 
+    def add_many(self, rules: Iterable[Rule]) -> None:
+        """Add many rules at ones.
+
+        :param rules: Iterable object of rules
+        """
+
+        hashes = set()
+
+        for rule in rules:
+            if rule.__hash__() in self._rule_hashes:
+                raise RuleSetConfigError(f"Rule '{rule.name}' was already configured in the RuleSet")
+
+            if rule.__hash__() in hashes:
+                raise RuleSetConfigError(f"Rule '{rule.name}' is duplicated on the given rules")
+
+            hashes.add(rule.__hash__())
+
+        self._rules.add_many(rules)
+        self._rule_hashes.update(hashes)
+
     def count_rules(self) -> int:
         """Count the total of rules configured on the set.
 
@@ -97,6 +125,19 @@ class RuleSet:
 
         return len(self._rules)
 
+    def rule_names(self) -> List[AnyStr]:
+        """Return a list with the names of all the configured rules of the set.
+
+        :return List[AnyStr]: List of rule names
+        """
+
+        names = []
+
+        for rule in self._rules:
+            names.append(rule.name)
+
+        return names
+
     def apply(self, data: Any, fail_fast: Optional[bool] = True) -> NoReturn:
         """Apply the configured rule set to a specific data.
 
@@ -154,13 +195,15 @@ class RuleSet:
 
 
 class Ruler:
-    """Compound ruler class."""
+    """Store RuleSet objects to be applied over a given data. This class can apply one single stored RuleSet,
+    a list of of some of the configured RuleSet objects or all of them.
+    """
 
     _rule_sets: Dict[AnyStr, RuleSet]
     _rule_set_hashes: Set[int]
 
     def __init__(self):
-        self._rule_sets = {}
+        self._rule_sets = dict()
         self._rule_set_hashes = set()
 
     def add_set(self, rule_set: RuleSet) -> NoReturn:
@@ -171,11 +214,33 @@ class Ruler:
         """
 
         if rule_set.__hash__() in self._rule_set_hashes:
-            raise RulerConfigError(f"RuleSet '{rule_set.name}' was already configured on the ruler", )
+            raise RulerConfigError(f"RuleSet '{rule_set.name}' was already configured on the ruler")
 
         self._rule_sets.update({rule_set.name: rule_set})
         self._rule_set_hashes.add(rule_set.__hash__())
 
+    def add_many(self, rule_sets: Iterable[RuleSet]) -> NoReturn:
+        """Add many RuleSet objects at ones.
+
+        :param rule_sets: Iterable object of rule sets to be added
+        """
+
+        hashes = set()
+        new_sets = dict()
+
+        for rule_set in rule_sets:
+            if rule_set.__hash__() in self._rule_set_hashes:
+                raise RulerConfigError(f"RuleSet '{rule_set.name}' was already configured on the ruler")
+
+            if rule_set.__hash__() in hashes:
+                raise RulerConfigError(f"RuleSet '{rule_set.name}' is duplicated on the given rule sets")
+
+            new_sets.update({rule_set.name: rule_set})
+            hashes.add(rule_set.__hash__())
+
+        self._rule_sets.update(new_sets)
+        self._rule_set_hashes.update(hashes)
+
     def count_sets(self) -> int:
         """Count the total of configured sets.
 
@@ -184,10 +249,18 @@ class Ruler:
 
         return len(self._rule_sets.keys())
 
+    def rule_set_names(self) -> List[AnyStr]:
+        """Return a list of all configured RuleSet objects in the ruler.
+
+        :return: List of rule set names
+        """
+
+        return list(self._rule_sets.keys())
+
     def apply(
         self,
-        sets: Union[AnyStr, Tuple, List[AnyStr], Set[AnyStr]],
         data: Any,
+        sets: Optional[Union[AnyStr, Tuple, List[AnyStr], Set[AnyStr]]] = None,
         fail_fast: Optional[bool] = True,
     ) -> NoReturn:
         """Apply specified rule set, all of the stored sets or a sub collection
@@ -210,8 +283,8 @@ class Ruler:
 
     def _apply_set(
         self,
-        set_names: Set[AnyStr],
         data: Any,
+        set_names: Set[AnyStr],
         fail_fast: Optional[bool] = True,
     ) -> NoReturn:
         """Apply configured rule sets to the provided data.
@@ -249,18 +322,18 @@ class Ruler:
         rule_set = self._rule_sets.get(set_name, None)
 
         if rule_set is None:
-            raise RulerError(f"Not found RuleSet '{set_name}' on Ruler configuration", )
+            raise RulerError(f"Not found RuleSet '{set_name}' on Ruler configuration")
 
         return rule_set
 
-    def _process_set_names(self, sets: Any) -> Union[AnyStr, Set[AnyStr]]:
+    def _process_set_names(self, sets: Optional[Any] = None) -> Union[AnyStr, Set[AnyStr]]:
         """Verify the set names that will be applied and format it.
 
         :param sets: Set name or List of set names.
         :return Union[AnyStr, Set[AnyStr]]: Formatted set names
         """
 
-        if isinstance(sets, str) and sets.lower() == 'all':
+        if sets is None:
             return set(self._rule_sets.keys())
 
         if isinstance(sets, (tuple, list)):
